//! AES-128 and square attack implementation in rust.
//!
//! ### commands
//! #### encrypt
//! encrypt or decrypt a text
//!
//! #### findkey
//! square attack against 4-turns AES-128 with a specific key
//!
//! #### square
//! square attack against 4-turns AES-128 with n randomly generated keys
//!
//! # Examples
//! encrypt the text "Hello, world!" using the key 2b7e151628aed2a6abf7158809cf4f3c, hex-encoded. 10-turns AES-128
//!```
//! $ cargo run -- encrypt -K -k '2b7e151628aed2a6abf7158809cf4f3c' -t "Hello, world!" -n 10
//!```
//! decrypt the text 7d6a80dde1d56639903194344eddf515, hex-encoded, using the key 2b7e151628aed2a6abf7158809cf4f3c, hex-encoded. 10-turns AES-128
//!```
//! $ cargo run -- encrypt -d -Kk '2b7e151628aed2a6abf7158809cf4f3c' -Tt '7d6a80dde1d56639903194344eddf515' -n 10
//!```
//! square attack against 4-turns AES-128, choosen key bba51a66aca801747294ff4317fb59e2, hex-encoded.
//!```
//! $ cargo run -- findkey bba51a66aca801747294ff4317fb59e2 -K
//!```
//! square attack against 4-turns AES-128, 10 keys generated randomly.
//!```
//! $ cargo run -- square 10
//!```

mod aes;
mod square;
mod utils;
use crate::aes::Aes;

use clap::{Parser, Subcommand};

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
#[command(propagate_version = true)]
struct Cli {
    #[command(subcommand)]
    command: Commands,
}

#[derive(Subcommand)]
enum Commands {
    /// Ecnrypt
    Encrypt {
        /// (default Some('11'))
        #[arg(long = "nturn", short = 'n')]
        nturn: Option<String>,
        /// (default "")
        #[arg(long = "text", short = 't')]
        text: String,
        /// (default True)
        #[arg(long = "hextext", short = 'T')]
        hextext: bool,
        /// (default "")
        #[arg(long = "key", short = 'k')]
        key: String,
        /// (default True)
        #[arg(long = "hexkey", short = 'K')]
        hexkey: bool,
        /// (default False)
        #[arg(long = "decrypt", short = 'd')]
        decrypt: bool,
    },
    Findkey {
        /// (default "")
        key: String,
        /// (default True)
        #[arg(long = "hexkey", short = 'K')]
        hexkey: bool,
    },
    Square {
        /// (default "1")
        number: String,
    },
}

fn main() -> std::io::Result<()> {
    let cli = Cli::parse();

    match &cli.command {
        Commands::Encrypt {
            nturn,
            text,
            hextext,
            key,
            hexkey,
            decrypt,
        } => {
            println!("original text: {}", &text);
            let result: [u8; 16];
            let mut aesnturn: usize = 10;

            let aeskey: [u8; 16] = match hexkey {
                false => utils::parse_hex_string(key),
                true => utils::string_to_u8_16(key),
            };
            if let Some(number) = nturn {
                aesnturn = number.parse::<usize>().unwrap()
            }

            let aescipher = Aes::new(&aeskey, &aesnturn);

            let aestext: [u8; 16] = match hextext {
                true => utils::parse_hex_string(text),
                false => utils::string_to_u8_16(text),
            };

            match decrypt {
                true => {
                    result = aescipher.decrypt_block(&aestext);
                    print!("Decrypted text, hex-encoded: ");
                }
                false => {
                    result = aescipher.encrypt_block(&aestext);
                    print!("Encrypted text, hex-encoded: ");
                }
            }
            utils::print_array(&result);
            println!();
        }
        Commands::Findkey { key, hexkey } => {
            let aeskey: [u8; 16] = match hexkey {
                false => utils::parse_hex_string(key),
                true => utils::string_to_u8_16(key),
            };
            let keyfound: [u8; 16] = Aes::findkey(&aeskey);
            if keyfound == aeskey {
                println!("key found.");
            } else {
                println!("key not found.");
            }
        }
        Commands::Square { number } => {
            Aes::test_square_attack(number.parse::<usize>().unwrap());
        }
    }
    Ok(())
}
